use indexed_view_online_test

-- alter links, add IsMigration flag with default 0 and constraint 0 
set statistics io, time on
ALTER TABLE dbo.LINKS 
ADD IsMigrated BIT NOT NULL CONSTRAINT DF_LINKS_IsMigrated DEFAULT 0;
GO -- 0 seconds

-- create indexed view
CREATE VIEW dbo.LINK_FULLURLS
WITH SCHEMABINDING
AS

SELECT 
  L.LinkId,
  CASE 
	WHEN L.IsExternal = 1 THEN L.URL
	ELSE C.BaseURL + L.URL
  END AS FullURL,
  CHECKSUM 
    ( 
      CASE 
	    WHEN L.IsExternal = 1 THEN L.URL
	    ELSE C.BaseURL + L.URL
      END
    ) AS FullURLChecksum
FROM dbo.LINKS L 
INNER JOIN dbo.CLIENTS C
  ON C.ClientId = L.ClientId
WHERE L.DateDeleted IS NULL
  AND L.IsMigrated = 1;
GO -- zero seconds

-- read some/most/all of CX into memory
DECLARE @Count BIGINT;
SELECT @Count = COUNT(*) 
FROM dbo.LINKS WITH (NOLOCK, INDEX(1));
GO -- 20 seconds online

CREATE UNIQUE CLUSTERED INDEX IX_LINK_FULLURLS
  ON dbo.LINK_FULLURLS(LinkId);
go -- parallel scan of data mostly in memory, no writes, 0.8 seconds offline

CREATE NONCLUSTERED INDEX IX_LINK_FULLURLS_FullURLChecksum
  ON dbo.LINK_FULLURLS(FullURLChecksum) 
  INCLUDE (FullURL)
  WITH (ONLINE = ON);
GO -- online and zero seconds

-- alter content_objects, remove default, change default to 1
BEGIN TRAN

  ALTER TABLE dbo.LINKS DROP CONSTRAINT DF_LINKS_IsMigrated;
  
  ALTER TABLE dbo.LINKS ADD CONSTRAINT DF_LINKS_IsMigrated DEFAULT 1 FOR IsMigrated;
  
COMMIT 
GO -- zero seconds

-- batch change isMigration to 1
set statistics io, time off;
set nocount on;

DECLARE
	@LargestLinkIdProcessed INT = -1,
	@NextBatchMax INT,
	@RC INT = 1;
 
WHILE (@RC > 0)
BEGIN
 
  SELECT TOP (1000) @NextBatchMax = LinkId
  FROM dbo.LINKS
  WHERE LinkId > @LargestLinkIdProcessed
    AND IsMigrated = 0
  ORDER BY LinkId ASC;
   
  UPDATE dbo.LINKS
  SET IsMigrated = 1
  WHERE LinkId > @LargestLinkIdProcessed
    AND LinkId <= @NextBatchMax;
 
  SET @RC = @@ROWCOUNT;
  SET @LargestLinkIdProcessed = @NextBatchMax;
 
END

GO -- 8 minutes, but online

